#include "utils.h"

#include <linux/mvf-fb.h>

#include <QFile>
#include <QDebug>

static QString pipeline_camera;
static QString cores;
static QComboBox *fileCombo;

/*
 * read the /etc/timesys-theatre.conf file
 * for video pipelines (many allowed) load the combo box
 * for the # of cores and camera pipeline just save it
 */
void load_config_file(QComboBox *combo)
{

    fileCombo = combo;

    pipeline_camera = "";
    cores = "";

    QFile config(CONFIG_NAME);
    config.open(QIODevice::ReadOnly);
    if (!config.isOpen())
    {
        qDebug() << "Could not open " << config.fileName() << endl;
        exit(1);
    }

    while (!config.atEnd())
    {

        QString line = config.readLine(512).trimmed();

        if (line.at(0) == '#') // comment
            continue;

        // key value airs sepearted by = sign
        int eq_idx = line.indexOf("=");
        if (eq_idx < 0)
        {
            qDebug() << "No '=' in config line. skipping: " << line << endl;
            continue;
        }

        QString key = line.left(eq_idx);
        QString val = line.right(line.length() - eq_idx - 1);

        if (key == "pipeline")
        {
            // add to the dropdown
            int semi_idx = val.indexOf(";");
            if (semi_idx < 0)
            {
                qDebug() << "No ';' in config pipeline line. skipping: " << line
                        << endl;
                continue;
            }

            QString desc = val.left(semi_idx);
            QString pipeline = val.right(val.length() - semi_idx - 1);

            combo->addItem(desc, pipeline);
        }
        else if (key == "pipeline_camera")
        {
            pipeline_camera = val;
        }
        else if (key == "cores")
        {
            cores = val;
        }
        else
        {
            qDebug() << "Bad key in config file: " << key << " skipping."
                    << endl;
        }
    }

    combo->setCurrentIndex(0);

    config.close();
}
/**
 * Search the program configuration for the value of a specified field
 * @param key	key to query value for
 * @param buffer	character buffer to which to copy value found
 * @return	true if operation successful, false if not
 */
bool get_config(QString key, char buffer[])
{
    buffer[0] = 0;
    int i;
    QString return_str;

    if (key == "pipeline")
        return_str = fileCombo->itemData(fileCombo->currentIndex()).toString();

    else if (key == "pipeline_camera")
        return_str = pipeline_camera;

    else if (key == "cores")
        return_str = cores;

    else
        return false;

    if (return_str.length() == 0)
        return false;

    for (i = 0; i < return_str.length(); i++)
        buffer[i] = return_str.at(i).toAscii();
    buffer[return_str.length()] = 0;

    return true;
}

// Set fb0/fb2 blend level
// 0 means no fb0; 255 means fully fb0
// Return true if operation was a success
// Note: call this AFTER gstreamer initializations: something in them resets the alpha
bool set_alpha(int alpha_arg)
{
    set_vybrid_alpha("/dev/fb0", alpha_arg);

    return true;
}

void blank_vybrid(char *dev)
{

    // if there's erros just leave silently - its not that important

    struct fb_var_screeninfo vinfo;
    struct fb_fix_screeninfo finfo;

    int fb = open(dev, O_RDWR);
    if (fb < 0)
        return;

    if (ioctl(fb, FBIOGET_FSCREENINFO, &finfo))
        return;

    if (ioctl(fb, FBIOGET_VSCREENINFO, &vinfo))
        return;

    unsigned int memsize = finfo.line_length * vinfo.yres;
    void *data = (void *) mmap(0, memsize, PROT_READ | PROT_WRITE, MAP_SHARED
            , fb, 0);
    if (data == (void *) -1)
        return;

    memset(data, 0, memsize);

    close(fb);
    munmap(data, memsize);
}

// Clears and wakes primary framebuffer (fb0)
// Returns true if successful
bool blank_fbs()
{
    blank_vybrid("/dev/fb0");
    blank_vybrid("/dev/fb2");

    return true;
}

bool set_vybrid_alpha(char *dev, unsigned char alpha)
{
    printf("dev: %s, alpha=%d\n", dev, alpha);
    int fb = open(dev, O_RDWR);
    if (fb < 0)
    {
        printf("\nFailed to open %s. Now exiting", dev);
        exit(-1);
    }

    if (ioctl(fb, MFB_SET_ALPHA, &alpha) < 0)
    {
        printf("\nFailed to set alpha for %s. Now exiting", dev);
        exit(-1);
    }

    close(fb);

    return true;
}

bool set_vybrid_layer(char *dev, int x, int y)
{

    int fb = open(dev, O_RDWR);
    if (fb < 0)
    {
        printf("\nFailed to open %s. Now exiting", dev);
        exit(-1);
    }

    struct layer_display_offset layer_d;
    layer_d.x_layer_d = x;
    layer_d.y_layer_d = y;
    if (ioctl(fb, MFB_SET_LAYER, &layer_d) < 0)
    {
        printf("\nFailed to set layer for %s. Now exiting", dev);
        exit(-1);
    }

    close(fb);

    return true;
}

/*
 * The i.mx6 version of this code uses fb2 for gstreamer, we need to
 * make sure fb1 is not in the way and fb2 is aligned properly
 * with fb0
 */
bool init_vybrid_fb()
{
    set_vybrid_layer("/dev/fb0", 0, 0);
    set_vybrid_alpha("/dev/fb0", 255);
    set_vybrid_layer("/dev/fb1", 0, 0);
    set_vybrid_alpha("/dev/fb1", 0);
    set_vybrid_layer("/dev/fb2", 0, 0);
    set_vybrid_alpha("/dev/fb2", 255);
    return true;
}

// Resizes fb2 overlay to the size of fb0
// Returns 'true' if successful
bool resize_fb2()
{
    struct fb_var_screeninfo fb0_screeninfo;
    struct fb_var_screeninfo fb2_screeninfo;

    int fb0_fd;
    int fb2_fd;

    int fb2_new_xres; // New resolutions for fb2 resolution
    int fb2_new_yres; // Based on fb0 resolutions

    // Open fb0 to read screen dimensions
    printf("\nOpening /dev/fb0 for reading");
    fb0_fd = open("/dev/fb0", O_RDONLY);
    if (fb0_fd < 0)
    {
        printf("\nFailed to open /dev/fb0. Now exiting");
        exit(-1);
    }

    // Open fb2 to read and set screen dimensions
    printf("\nOpening /dev/fb2 for reading and writing");
    fb2_fd = open("/dev/fb2", O_RDWR);
    if (fb2_fd < 0)
    {
        printf("\nFailed to open /dev/fb2. Now exiting");
        exit(-1);
    }

    // Read fb0 screen dimensions
    printf("\nReading /dev/fb0 variable screen information");
    if (ioctl(fb0_fd, FBIOGET_VSCREENINFO, &fb0_screeninfo) < 0)
    {
        printf("\nFailed to obtain fb0 variable screen info. Now exiting");
        exit(-1);
    }

    printf("\nfb0 xres: %d", fb0_screeninfo.xres);
    printf("\nfb0 yres: %d", fb0_screeninfo.yres);

    // Set fb2 new resolutions
    fb2_new_xres = fb0_screeninfo.xres;
    fb2_new_yres = fb0_screeninfo.yres;

    printf("\nClosing fb0 file descriptor");
    close(fb0_fd);

    // Obtain fb2 variable screen info
    printf("\nObtaining /dev/fb2 variable screen information");
    if (ioctl(fb2_fd, FBIOGET_VSCREENINFO, &fb2_screeninfo) < 0)
    {
        printf("\nFailed to obtain fb2 variable screen info. Now exiting");
        exit(-1);
    }

    // Set fb2 variable screen info based on new fb2 resolutions
    printf("\nSetting fb2 resolutions to match fb0");
    fb2_screeninfo.xres = fb2_new_xres;
    fb2_screeninfo.yres = fb2_new_yres;

    // Write new resolution values to fb2
    printf("\nAttempting to \'put\' new values of xres and yres to fb2");
    if (ioctl(fb2_fd, FBIOPUT_VSCREENINFO, &fb2_screeninfo) < 0)
    {
        printf("\nError writing new resolution values to fb2. Now exiting");
        exit(-1);
    }

    // Close fb2 file descriptor, clean up
    printf("\nClosing fb2 file descriptor");
    printf("\n");
    close(fb2_fd);

    return true;
}
